/**
    example03b.java  Verify a secret on a SmartDongle written using the utility
                    in Example3a

    MicroWorks, Inc.
    2808 North Cole Road
    Boise, ID 83704

    www.smartdongle.com


    ***** BEGIN LICENSE BLOCK *****

    Version: MPL 1.1/GPL 2.0/LGPL 2.1

    The contents of this file are subject to the Mozilla Public License
    Version 1.1 (the "License"); you may not use this file except in
    compliance with the License. You may obtain a copy of the License at
    http://www.mozilla.org/MPL/

    Software distributed under the License is distributed on an "AS IS" basis,
    WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
    for the specific language governing rights and limitations under the
    License.

    The Original Code is MicroWorks, Inc. code.

    The Initial Developer of the Original Code is MicroWorks, Inc. Portions
    created by MicroWorks, Inc. are Copyright (C) 2010
    MicroWorks, Inc. All Rights Reserved.

    Contributor(s):

    Alternatively, the contents of this file may be used under the terms of
    either the GNU General Public License Version 2 or later (the "GPL"), or
    the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
    in which case the provisions of the GPL or the LGPL are applicable instead
    of those above. If you wish to allow use of your version of this file only
    under the terms of either the GPL or the LGPL, and not to allow others to
    use your version of this file under the terms of the MPL, indicate your
    decision by deleting the provisions above and replace them with the notice
    and other provisions required by the GPL or the LGPL. If you do not delete
    the provisions above, a recipient may use your version of this file under
    the terms of any one of the MPL, the GPL or the LGPL.

    ***** END LICENSE BLOCK *****
*/
package com.microworks;

import javax.crypto.Cipher;
import javax.crypto.spec.SecretKeySpec;
import com.microworks.SmartDongle;

public class Example3b {
  private static int error;

  // Your keys here
  private final static long p1 = 0xEC6CC589AEFD1E75L;
  private final static long p2 = 0xFCEC0A6A82747B3FL;

  // Encrypted data stored on SmartDongle.
  private final static String plainText = "This is the secret written to SmartDongle, it can be passwords, serial numbers, program parameters, etc";

  private final static int aesBlockSize = 128;

  public static void main(String args[]) {
    /*
        First, calculate the number of bytes you will need to read from the
        SmartDongle since the encrypted bytes are rounded up to the nearest
        multiple AES block size.
    */
    int plainTextLength = plainText.length();

    int cipherTextBlocks = plainTextLength / (aesBlockSize / 8);
    if (plainTextLength % (aesBlockSize / 8) != 0) {
        cipherTextBlocks++;
    }

    // Total cipher length to be read from SmartDongle.
    int cipherTextLength = cipherTextBlocks * (aesBlockSize / 8);

    try {
      /*
        This instantiates an AES key read from SmartDongle written in
        Example3a. This key is used to initialize an AES cipher for decryption.

        Note that you could simply store the key in this class as
        private final static byte[] AESkey = {
          'y','o','u','r',' ','1','6', ' ','b','y','t','e',' ','k','e','y'}

        Given the ease of which Java is decompliled, storing the key outside
        of the application is a good ideas since this makes for an extra level
        of obfuscation.
      */

      // Retrieve the AES key from the SmartDongle
      byte[] AESkey = new byte[aesBlockSize / 8];
      int address = 0;
      error = SmartDongle.Read(p1,
                              p2,
                              address + cipherTextLength,
                              AESkey,
                              aesBlockSize / 8);
      if (error != 0) {
        // FAIL exiting...
        return;
      }

      /*
         This instantiates a AES key retrieved from the SmartDongle
         key is used to initialize a AES cipher for decryption.
      */
      SecretKeySpec skeySpec = new SecretKeySpec(AESkey, "AES");
      Cipher cipher = Cipher.getInstance("AES");
      cipher.init(Cipher.DECRYPT_MODE, skeySpec);

      // Retrieve users encrypted application data
      byte[] input = new byte[cipherTextLength];
      error = SmartDongle.Read(p1,
                              p2,
                              address,
                              input,
                              cipherTextLength);
      if (error != 0) {
        // FAIL exiting...
        return;
      }

      // Decrypt the user's application data
      byte[] decryptedBytes = cipher.doFinal(input);

      /*
        Verify the applicaton data that was written in Example3a
        Note: This is just an excercise, a developer could do any number
        of things with the application data here.
      */
      int i;
      byte[] plainTextBytes = plainText.getBytes();
      for (i = 0; i < plainText.length(); i++) {
        if (decryptedBytes[i] != plainTextBytes[i] ) {
           // FAIL exiting...
           return;
        }
      }
    }
    catch (java.security.NoSuchAlgorithmException nsafe) {
      System.out.println("Caught Exception:  " + nsafe);
    }
    catch (javax.crypto.NoSuchPaddingException nspe) {
      System.out.println("Caught Exception:  " + nspe);
    }
    catch (javax.crypto.BadPaddingException bpe) {
      System.out.println("Caught Exception:  " + bpe);
    }
    catch (javax.crypto.IllegalBlockSizeException ibse) {
      System.out.println("Caught Exception:  " + ibse);
    }
    catch (java.security.InvalidKeyException ike) {
      System.out.println("Caught Exception:  " + ike);
    }

    /*
       Your main class code goes here
    */
    System.out.println("Success: Will execute main class");
  }
}
